#!/bin/sh

# quit on errors:
set -o errexit

# quit on unbound symbols:
set -o nounset

DIR=`dirname "$0"`

cd $DIR
export FLASK_APP=app.py

# Load fixtures
flask fixtures records
flask index reindex --yes-i-know -t recid
flask index run
